// Copyright Epic Games, Inc. All Rights Reserved.

#pragma once

// Enable to constrain clusters to no more than 256 vertices and no index references outside of trailing window of NANITE_CONSTRAINED_CLUSTER_CACHE_SIZE vertices.
#define NANITE_USE_CONSTRAINED_CLUSTERS						1

#define NANITE_USE_UNCOMPRESSED_VERTEX_DATA					0

#define NANITE_MAX_STREAMING_REQUESTS						(256u * 1024u)
#define NANITE_MAX_CLUSTER_TRIANGLES						128
#define NANITE_MAX_CLUSTER_VERTICES_BITS					8
#define NANITE_MAX_CLUSTER_VERTICES							(1 << NANITE_MAX_CLUSTER_VERTICES_BITS)
#define NANITE_MAX_CLUSTER_VERTICES_MASK					((1 << NANITE_MAX_CLUSTER_VERTICES_BITS) - 1)

#define NANITE_MAX_CLUSTER_INDICES							(NANITE_MAX_CLUSTER_TRIANGLES * 3)
#define NANITE_MAX_UVS										4

#define NANITE_MAX_CLUSTER_MATERIALS						64

#define NANITE_USE_STRIP_INDICES							1

#define NANITE_ROOT_PAGE_GPU_SIZE_BITS						15
#define NANITE_ROOT_PAGE_GPU_SIZE							(1u << NANITE_ROOT_PAGE_GPU_SIZE_BITS)

// Adds frame counter dependent magic values to streaming requests to help verify their integrity.
#define NANITE_SANITY_CHECK_STREAMING_REQUESTS				0

#define NANITE_STREAMING_REQUEST_MAGIC_BITS					8
#define NANITE_STREAMING_REQUEST_MAGIC_MASK					((1 << NANITE_STREAMING_REQUEST_MAGIC_BITS) - 1)

#define NANITE_STREAMING_PAGE_GPU_SIZE_BITS					17
#define NANITE_STREAMING_PAGE_GPU_SIZE						(1u << NANITE_STREAMING_PAGE_GPU_SIZE_BITS)
#define NANITE_MAX_PAGE_DISK_SIZE							(NANITE_STREAMING_PAGE_GPU_SIZE * 2)	

#define NANITE_MAX_CLUSTERS_PER_PAGE_BITS					8
#define NANITE_MAX_CLUSTERS_PER_PAGE_MASK					((1 << NANITE_MAX_CLUSTERS_PER_PAGE_BITS) - 1)
#define NANITE_MAX_CLUSTERS_PER_PAGE						( 1 << NANITE_MAX_CLUSTERS_PER_PAGE_BITS)
#define NANITE_MAX_CLUSTERS_PER_GROUP_BITS					9
#define NANITE_MAX_CLUSTERS_PER_GROUP_MASK					((1 << NANITE_MAX_CLUSTERS_PER_GROUP_BITS) - 1)
#define NANITE_MAX_CLUSTERS_PER_GROUP						((1 << NANITE_MAX_CLUSTERS_PER_GROUP_BITS) - 1)
#define NANITE_MAX_CLUSTERS_PER_GROUP_TARGET				128

#define NANITE_MAX_HIERACHY_CHILDREN_BITS					6
#define NANITE_MAX_HIERACHY_CHILDREN						(1 << NANITE_MAX_HIERACHY_CHILDREN_BITS)
#define NANITE_MAX_VIEWS_PER_CULL_RASTERIZE_PASS_BITS		12
#define NANITE_MAX_VIEWS_PER_CULL_RASTERIZE_PASS_MASK		((1 << NANITE_MAX_VIEWS_PER_CULL_RASTERIZE_PASS_BITS) - 1)
#define NANITE_MAX_VIEWS_PER_CULL_RASTERIZE_PASS			(1 << NANITE_MAX_VIEWS_PER_CULL_RASTERIZE_PASS_BITS)
#define NANITE_MAX_GPU_PAGES_BITS							16
#define NANITE_MAX_GPU_PAGES_MASK							((1 << NANITE_MAX_GPU_PAGES_BITS) - 1)
#define NANITE_MAX_GPU_PAGES								( 1 << NANITE_MAX_GPU_PAGES_BITS)
#define NANITE_MAX_INSTANCES_BITS							24
#define NANITE_MAX_INSTANCES								(1 << NANITE_MAX_INSTANCES_BITS)	

#define NANITE_MAX_NODES_PER_PRIMITIVE_BITS					16
#define NANITE_MAX_RESOURCE_PAGES_BITS						20
#define NANITE_MAX_RESOURCE_PAGES_MASK						((1 << NANITE_MAX_RESOURCE_PAGES_BITS) - 1)
#define NANITE_MAX_RESOURCE_PAGES							(1 << NANITE_MAX_RESOURCE_PAGES_BITS)
#define NANITE_MAX_GROUP_PARTS_BITS							3
#define NANITE_MAX_GROUP_PARTS_MASK							((1 << NANITE_MAX_GROUP_PARTS_BITS) - 1)
#define NANITE_MAX_GROUP_PARTS								(1 << NANITE_MAX_GROUP_PARTS_BITS)

#define NANITE_MAX_BVH_NODE_FANOUT_BITS						2
#define NANITE_MAX_BVH_NODE_FANOUT_MASK						((1 << NANITE_MAX_BVH_NODE_FANOUT_BITS)-1)
#define NANITE_MAX_BVH_NODE_FANOUT							(1 << NANITE_MAX_BVH_NODE_FANOUT_BITS)
#define NANITE_BVH_NODE_ENABLE_MASK							((1 << NANITE_MAX_BVH_NODE_FANOUT)-1)

#define NANITE_PERSISTENT_CLUSTER_CULLING_GROUP_SIZE		64
#define NANITE_MAX_BVH_NODES_PER_GROUP						(NANITE_PERSISTENT_CLUSTER_CULLING_GROUP_SIZE / NANITE_MAX_BVH_NODE_FANOUT)

#define NANITE_NUM_STREAMING_PRIORITY_CATEGORY_BITS			2
#define NANITE_STREAMING_PRIORITY_CATEGORY_MASK				((1 << NANITE_NUM_STREAMING_PRIORITY_CATEGORY_BITS) - 1)

#define NANITE_VIEW_FLAG_HZBTEST							0x1

#define NANITE_MAX_STATE_BUCKET_ID							((1 << 14) - 1)

#define NANITE_CLUSTER_FLAG_LEAF							0x1

#define NANITE_MAX_TRANSCODE_GROUPS_PER_PAGE				128

#define NANITE_NUM_PACKED_CLUSTER_FLOAT4S					6
#define NANITE_GPU_PAGE_HEADER_SIZE							16

#define NANITE_MAX_POSITION_QUANTIZATION_BITS				21	// (21*3 = 63) < 64
#define NANITE_POSITION_PRECISION_BITS						5
#define NANITE_MIN_POSITION_PRECISION						-8
#define NANITE_MAX_POSITION_PRECISION						23

#define NANITE_NORMAL_QUANTIZATION_BITS						9

#define NANITE_MAX_TEXCOORD_QUANTIZATION_BITS				15
#define NANITE_MAX_COLOR_QUANTIZATION_BITS					8

#define NANITE_NUM_CULLING_FLAG_BITS						3
#define NANITE_CULLING_FLAG_TEST_LOD						0x1
#define NANITE_CULLING_FLAG_USE_HW							0x2
#define NANITE_CULLING_FLAG_FROM_DISOCCLUDED_INSTANCE		0x4

#define NANITE_VERTEX_COLOR_MODE_WHITE						0
#define NANITE_VERTEX_COLOR_MODE_CONSTANT					1
#define NANITE_VERTEX_COLOR_MODE_VARIABLE					2

#define NANITE_PAGE_FLAG_RELATIVE_ENCODING					0x1

#define NANITE_INVALID_PERSISTENT_HASH						0

#define NANITE_RESOURCE_FLAG_HAS_VERTEX_COLOR				0x1
#define NANITE_RESOURCE_FLAG_HAS_IMPOSTER					0x2
#define NANITE_RESOURCE_FLAG_STREAMING_DATA_IN_DDC			0x4

#define NANITE_RENDER_FLAG_HAVE_PREV_DRAW_DATA				0x1
#define NANITE_RENDER_FLAG_FORCE_HW_RASTER					0x2
#define NANITE_RENDER_FLAG_PRIMITIVE_SHADER					0x4
#define NANITE_RENDER_FLAG_MESH_SHADER						0x8
#define NANITE_RENDER_FLAG_OUTPUT_STREAMING_REQUESTS		0x10
#define NANITE_RENDER_FLAG_REVERSE_CULLING					0x20
#define NANITE_RENDER_FLAG_IS_SCENE_CAPTURE					0x40
#define NANITE_RENDER_FLAG_IS_REFLECTION_CAPTURE			0x80
#define NANITE_RENDER_FLAG_IS_GAME_VIEW						0x100
#define NANITE_RENDER_FLAG_GAME_SHOW_FLAG_ENABLED			0x200
#define NANITE_RENDER_FLAG_EDITOR_SHOW_FLAG_ENABLED			0x400
#define NANITE_RENDER_FLAG_IS_LUMEN_CAPTURE					0x800

// Only available with the DEBUG_FLAGS permutation active.
// Default value (no debug) is 0
#define NANITE_DEBUG_FLAG_WRITE_STATS						0x1
#define NANITE_DEBUG_FLAG_DISABLE_CULL_HZB_BOX				0x2
#define NANITE_DEBUG_FLAG_DISABLE_CULL_HZB_SPHERE			0x4
#define NANITE_DEBUG_FLAG_DISABLE_CULL_FRUSTUM_BOX			0x8
#define NANITE_DEBUG_FLAG_DISABLE_CULL_FRUSTUM_SPHERE		0x10
#define NANITE_DEBUG_FLAG_DRAW_ONLY_VSM_INVALIDATING		0x20
